/* include/hpdf_config.h.in.  Generated from configure.in by autoheader.  */

/* KITWARE_LIBHARU_CHANGE Include the symbol name mangler: */
#include "vtk_haru_mangle.h"

/* Define to 1 if you have the <dlfcn.h> header file. */
#define LIBHPDF_HAVE_DLFCN_H

/* Define to 1 if you have the <inttypes.h> header file. */
#define LIBHPDF_HAVE_INTTYPES_H

/* Define to 1 if you have the `png' library (-lpng). */
#define LIBHPDF_HAVE_LIBPNG

/* Define to 1 if you have the `z' library (-lz). */
#define LIBHPDF_HAVE_LIBZ

/* Define to 1 if you have the <memory.h> header file. */
#define LIBHPDF_HAVE_MEMORY_H

/* Define to 1 if you have the <stdint.h> header file. */
#define LIBHPDF_HAVE_STDINT_H

/* Define to 1 if you have the <stdlib.h> header file. */
#define LIBHPDF_HAVE_STDLIB_H

/* Define to 1 if you have the <strings.h> header file. */
#define LIBHPDF_HAVE_STRINGS_H

/* Define to 1 if you have the <string.h> header file. */
#define LIBHPDF_HAVE_STRING_H

/* Define to 1 if you have the <sys/stat.h> header file. */
#define LIBHPDF_HAVE_SYS_STAT_H

/* Define to 1 if you have the <sys/types.h> header file. */
#define LIBHPDF_HAVE_SYS_TYPES_H

/* Define to 1 if you have the <unistd.h> header file. */
#define LIBHPDF_HAVE_UNISTD_H

/* debug build */
/* #undef LIBHPDF_DEBUG */

/* debug trace enabled */
/* #undef LIBHPDF_DEBUG_TRACE */

/* libpng is not available */
/* #undef LIBHPDF_HAVE_NOPNGLIB */
/* #undef HPDF_NOPNGLIB */

/* zlib is not available */
/* #undef LIBHPDF_HAVE_NOZLIB */

/* Define to the address where bug reports for this package should be sent. */
#define LIBHPDF_PACKAGE_BUGREPORT "TODO"

/* Define to the full name of this package. */
#define LIBHPDF_PACKAGE_NAME "libHaru-2.2.0-gcc"

/* Define to the full name and version of this package. */
#define LIBHPDF_PACKAGE_STRING "libHaru-2.2.0-gcc"

/* Define to the one symbol short name of this package. */
#define LIBHPDF_PACKAGE_TARNAME "TODO"

/* Define to the version of this package. */
#define LIBHPDF_PACKAGE_VERSION "2.2.0"

/* Define to 1 if you have the ANSI C header files. */
#define LIBHPDF_STDC_HEADERS

/* Define to `unsigned int' if <sys/types.h> does not define. */
/* #undef LIBHPDF_size_t */
